/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001-02 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.core.internals;

import java.sql.*;
import java.util.*;
import org.apache.log4j.*;
import org.w3c.dom.*;
import com.silverwrist.util.*;
import com.silverwrist.venice.db.*;
import com.silverwrist.venice.except.*;
import com.silverwrist.venice.svc.internal.GlobalSite;
import com.silverwrist.venice.util.ServiceProvider;
import com.silverwrist.venice.util.MailSend;
import com.silverwrist.venice.util.XMLLoader;

public class GlobalSiteImpl implements GlobalSite
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Category logger = Category.getInstance(GlobalSiteImpl.class);

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private EngineBackend engine = null;             // the engine back end
  private DataPool datapool = null;                // the database pool
  private Properties email_props = null;           // email properties
  private javax.mail.Session mailsession = null;   // email session object
  private StockMessages stock_messages = null;     // stock messages holder

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  public GlobalSiteImpl(EngineBackend engine, Element config) throws ConfigException
  {
    this.engine = engine;
    XMLLoader loader = XMLLoader.get();

    // Get the <database/> section.
    DOMElementHelper config_h = new DOMElementHelper(config);
    Element sect = loader.configGetSubSection(config_h,"database");

    try
    { // allocate the data pool object
      datapool = new DataPool(sect);

    } // end try
    catch (SQLException e)
    { // could not initialize the SQL database!  this is a big fat error!
      logger.fatal("data pool init failed - " + e.getMessage(),e);
      throw new ConfigException("database access initialization failed - " + e.getMessage(),e);

    } // end catch

    // Get the <email/> section.
    sect = loader.configGetSubSection(config_h,"email");
    DOMElementHelper sect_h = new DOMElementHelper(sect);

    // initialize the email properties and get a mail session object
    email_props = new Properties();
    email_props.put("mail.transport.protocol","smtp");
    email_props.put("mail.smtp.host",sect_h.getSubElementText("smtp-host"));
    email_props.put("mail.from",sect_h.getSubElementText("mail-from-addr"));
    email_props.put("com.silverwrist.venice.email.mailer",sect_h.getSubElementText("mailer"));
    mailsession = javax.mail.Session.getInstance(email_props,null);

    // Get the <messages/> section.
    sect = loader.configGetSubSection(config_h,"messages");

    // Initialize the stock messages list.
    stock_messages = new StockMessages(sect);

  } // end class GlobalSiteImpl

  /*--------------------------------------------------------------------------------
   * Implementations from interface ServiceProvider
   *--------------------------------------------------------------------------------
   */

  public Object queryService(Class klass)
  {
    if (klass==DataPool.class)
      return datapool;
    if (klass==MailSend.class)
      return new BaseMailSend(this,email_props,mailsession);
    throw new NoSuchServiceException("GlobalSite",klass);

  } // end queryService

  public Object queryService(Class klass, String serviceid)
  {
    return this.queryService(klass);

  } // end queryService

  /*--------------------------------------------------------------------------------
   * Implementations from interface GlobalSite
   *--------------------------------------------------------------------------------
   */

  public Connection getConnection(String db_selector) throws SQLException
  {
    // db_selector is ignored for now
    return datapool.getConnection();

  } // end getConnection

  public String getStockMessage(String key)
  {
    return stock_messages.get(key);

  } // end getStockMessage

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public void shutdown()
  {
    if (datapool!=null)
      datapool.closeAllConnections();

  } // end shutdown

} // end class GlobalSiteImpl
